package ch05_javagrundlagen.fileio;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.time.LocalDate;
import java.util.Objects;

/**
 * Beispielprogramm zur Verarbeitung von Person-Objekten mit einfachen
 * File-Streams. Achtung: Hier noch ohne Ressourcen-Handling.
 * 
 * @author Michael Inden
 * 
 * Copyright 2011, 2014, 2017, 2020 by Michael Inden
 */
public final class PersonStreamDataStream 
{
	public static void main(final String[] args) throws Exception 
	{
		final String tmpDir = System.getProperty("java.io.tmpdir");
		final File file = new File(tmpDir, "Person.ser1");
		final String filepath = file.getAbsolutePath();
		final Person max = new Person("Max", LocalDate.now(), "Hamburg");

		// Speichere Person 'Max' in einer Datei
		System.out.println("Writing Person " + max + " to file '" + filepath + "'");
		writePersonToStream(max, new DataOutputStream(new FileOutputStream(file)));

		// Person aus der Datei einlesen
		final Person newPerson = readPersonFromStream(new DataInputStream(new FileInputStream(file)));
		System.out.println("Person " + newPerson + " from file '" + filepath + "'");

		// Prfe auf inhaltliche Gleichheit
		System.out.println("Gleich? " + max.equals(newPerson));
	}

	public static void writePersonToStream(final Person person, final DataOutputStream dataOutStream)
			throws IOException {
		dataOutStream.writeUTF(person.getName());
		dataOutStream.writeUTF(person.getCity());
		dataOutStream.writeUTF(person.getBirthday().toString());
		dataOutStream.flush();
	}

	public static Person readPersonFromStream(final DataInputStream dataInStream) throws IOException {
		final String name = dataInStream.readUTF();
		final String city = dataInStream.readUTF();
		final LocalDate birthday = LocalDate.parse(dataInStream.readUTF());

		return new Person(name, birthday, city);
	}

	public static final class Person {
		private final String name;
		private final LocalDate birthday;
		private final String city;

		public Person(final String name, final LocalDate birthday, final String city) {
			if (name == null || birthday == null || city == null)
				throw new IllegalArgumentException("parameters 'name', 'birthday' and 'city' must not be null!");

			this.name = name;
			this.birthday = birthday;
			this.city = city;
		}

		public final String getName() {
			return name;
		}

		public final LocalDate getBirthday() {
			return birthday;
		}

		public final String getCity() {
			return city;
		}

		public boolean equals(Object other) {
			if (other == null) // Null-Akzeptanz
				return false;

			if (this == other) // reflexive
				return true;

			if (this.getClass() != other.getClass()) // same type ?
				return false;

			final Person otherPerson = (Person) other;
			return this.getName().equals(otherPerson.getName()) && this.getCity().equals(otherPerson.getCity())
					&& this.getBirthday().equals(otherPerson.getBirthday());
		}

		public String toString() {
			final StringBuffer buf = new StringBuffer();

			buf.append("Person: ");

			buf.append("Name='");
			buf.append(getName());
			buf.append("' ");
			buf.append("City='");
			buf.append(getCity());
			buf.append("' ");
			buf.append("Birthday='");
			buf.append(getBirthday());
			buf.append("'");

			return buf.toString();
		}

		// Vermeide FindBugs Warning, obwohl diese Klasse niemals in Hash-Container
		// verwendet wird
		@Override
		public int hashCode() {
			return Objects.hash(this.name, this.birthday, this.city);
		}
	}
}
