package ch12_advancedjava.serializable.versioning;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.time.LocalDate;

import ch05_javagrundlagen.basics.EqualsUtils;

/**
 * Beispielprogramm fr die Bcher "Java Profi 5 / Java 9 -- Die Neuerungen"
 * 
 * @author Michael Inden
 * 
 * Copyright 2017 by Michael Inden 
 */
public final class PersonVersion1 implements Serializable
{
   // durch beliebige, immer gleichbleibende Kennung  
   // Berechnungsautomatismus deaktivieren  
   private static final long serialVersionUID = 1L;

   // $\mbox{\bfseries eigene Kennung der Klassenversion  
   private static final long CLASS_VERSION = 1L;
    
   private transient String  name;
   private transient String  city;
   private transient LocalDate    birthday;

   public PersonVersion1(final String name, final String city, 
                         final LocalDate birthday)
   {
      this.name = name;
      this.city = city;
      this.birthday = birthday;
   }

   private void writeObject(final ObjectOutputStream outStream) throws IOException
   {
      // eigene Versionsinformation schreiben 
      outStream.writeLong(CLASS_VERSION);

      // Attributdaten schreiben 
      outStream.writeUTF(name);
      outStream.writeUTF(city);
      outStream.writeObject(birthday);
   }

   private void readObject(final ObjectInputStream inStream) throws IOException, 
                                                          ClassNotFoundException
   {
      // eigene Versionsinformation lesen 
      final long version = inStream.readLong();

      if (version == CLASS_VERSION)
      {
          // Attributdaten lesen 
          name = inStream.readUTF();
          city = inStream.readUTF();
          birthday = (LocalDate) inStream.readObject();
      }
      else
      {
          // Diese Version untersttzt keine anderen Formate  
         throw new ClassNotFoundException("Unsupported version " + version);
       }
   }
   // ...

    @Override
    public String toString()
    {
        return super.toString() + "\n" + "[name=" + name + ", city=" + city + ", birthday=" + birthday + "]";
    }
    
    public final String getName()
    {
        return name;
    }

    public final String getCity()
    {
        return city;
    }

    public final LocalDate getBirthDay()
    {
        return birthday;
    }

    @Override
	public boolean equals(final Object other)
    {
        if (other == null) // null safe
            return false;

        if (this == other) // reflexive
            return true;

        if (this.getClass() != other.getClass())
            return false;

        final PersonVersion1 otherPerson = (PersonVersion1) other;
        return equalsImpl(otherPerson);
    }

    // Ansatz 1: geht immer gut, da value.equals(null) => false
    private boolean equalsImpl(final PersonVersion1 otherPerson)
    {
        return EqualsUtils.nullSafeEquals(this.getName(), otherPerson.getName())
               && EqualsUtils.nullSafeEquals(this.getCity(), otherPerson.getCity())
               && EqualsUtils.nullSafeEquals(this.getBirthDay(), otherPerson.getBirthDay());
    }
}