package ch12_advancedjava.serializable.versioning;

import java.awt.Color;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.time.LocalDate;

import ch05_javagrundlagen.basics.EqualsUtils;

/**
 * Beispielprogramm fr die Bcher "Java Profi 5 / Java 9 -- Die Neuerungen"
 * 
 * @author Michael Inden
 * 
 * Copyright 2017 by Michael Inden 
 */
public final class PersonVersion2 implements Serializable
{
    /** beliebige immer gleichbleibende Kennung */
    private static final long serialVersionUID = 1L;
    private static final long CLASS_VERSION = 2L;
    
    private String            name;
    private String            city;
    private transient LocalDate birthday;
    private transient Color   eyeColor;

    public PersonVersion2(final String name, final String city, final LocalDate birthday, final Color eyeColor)
    {
        this.name = name;
        this.city = city;
        this.birthday = birthday;
        this.eyeColor = eyeColor;
    }

    @Override
    public String toString()
    {
        return super.toString() + "\n" + "[name=" + name + ", city=" + city + ", birthday=" + birthday + ", eyeColor="
               + eyeColor + "]";
    }

	private void writeObject(final ObjectOutputStream outStream) throws IOException
	{
	    // Versionsinformation schreiben  
	    outStream.writeLong(CLASS_VERSION);
	
	    // Daten 
	    outStream.writeUTF(name);
	    outStream.writeUTF(city);
	
	    // Optimierte Darstellung fr LocalDate  
	    outStream.writeUTF(birthday.toString());
	
	    // Optimierte Darstellung fr Color  
	    outStream.writeInt(eyeColor.getRed());
	    outStream.writeInt(eyeColor.getGreen());
	    outStream.writeInt(eyeColor.getBlue());
	}
	
	private void readObject(final ObjectInputStream inStream) throws IOException, ClassNotFoundException
	{
	    // Versionsinformation ermitteln 
	    final long version = inStream.readLong();
	
	    // versionsabhngiges Einlesen  
	    if (version == CLASS_VERSION)
	    {
	        readDataInCurrentVersion(inStream);
	    }
	    else if (version == 1)
	    {
	        readDataOfVersion1(inStream);
	    }
	    else
	    {
	        throw new ClassNotFoundException("Unsupported version " + version);
	    }
	}
	
	private void readDataOfVersion1(final ObjectInputStream inStream) throws IOException, ClassNotFoundException 
	{
		name = inStream.readUTF();
		city = inStream.readUTF();
		birthday = (LocalDate) inStream.readObject();                
		eyeColor = Color.BLUE; // Vereinfachung fr Beispiel: Defaultwert Blau  
	}
	
	private void readDataInCurrentVersion(ObjectInputStream inStream) throws IOException 
	{
		name = inStream.readUTF();
		city = inStream.readUTF();
	
		final String birthdayAsText = inStream.readUTF();
		birthday = LocalDate.parse(birthdayAsText);
	
		final int red = inStream.readInt();
		final int green = inStream.readInt();
		final int blue = inStream.readInt();
		eyeColor = new Color(red, green, blue);
	}
	// ...

    public final String getName()
    {
        return name;
    }

    public final String getCity()
    {
        return city;
    }

    public final LocalDate getBirthDay()
    {
        return birthday;
    }

    public final Color getEyeColor()
    {
        return eyeColor;
    }

    @Override
	public boolean equals(final Object other)
    {
        if (other == null) // null safe
            return false;

        if (this == other) // reflexive
            return true;

        if (this.getClass() != other.getClass())
            return false;

        final PersonVersion2 otherPerson = (PersonVersion2) other;
        return equalsImpl(otherPerson);
    }

    // Ansatz 1: geht immer gut, da value.equals(null) => false
    private boolean equalsImpl(final PersonVersion2 otherPerson)
    {
        return EqualsUtils.nullSafeEquals(this.getName(), otherPerson.getName())
               && EqualsUtils.nullSafeEquals(this.getCity(), otherPerson.getCity())
               && EqualsUtils.nullSafeEquals(this.getBirthDay(), otherPerson.getBirthDay())
               && EqualsUtils.nullSafeEquals(this.getEyeColor(), otherPerson.getEyeColor());
    }
}