package ch08_dateandtime.temporaladjusters;

import static org.junit.Assert.assertEquals;

import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.Month;
import java.time.temporal.Temporal;
import java.time.temporal.TemporalAdjuster;

import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.CsvSource;

/**
 * Rudimentrer Unit Test fr die Klasse NextPaydayAdjuster.
 *
 * @author Michael Inden
 *
 * Copyright 2015, 2020 by Michael Inden
 */
public class NextPaydayAdjusterTest
{
    @Test
    public void checkAdvanceToPayDay_ForNormalMonthAndDaysBefore25th()
    {
        // Given
        final LocalDate day = LocalDate.of(2015, Month.MARCH, 26);
        final LocalDate expected = LocalDate.of(2015, Month.APRIL, 24);
        // When
        final LocalDate payday = day.with(new NextPaydayAdjuster());
        // Then
        assertEquals("should move to 25th or friday before", expected, payday);
        assertEquals(DayOfWeek.FRIDAY, payday.getDayOfWeek());
    }

    @Test
    public void checkAdvanceToPayDay_SpecialHandlingForDecember()
    {
        final LocalDate day = LocalDate.of(2014, Month.DECEMBER, 10);
        final LocalDate expected = LocalDate.of(2014, Month.DECEMBER, 15);

        final LocalDate payday = day.with(new NextPaydayAdjuster());

        assertEquals("should move to monday after 15th", expected, payday);
        assertEquals(DayOfWeek.MONDAY, payday.getDayOfWeek());
    }

    @Test
    public void checkAdvanceToNextMonth_ForDaysAfter25th()
    {
        final LocalDate day = LocalDate.of(2015, Month.MARCH, 26);
        final LocalDate expected = LocalDate.of(2015, Month.APRIL, 24);

        final LocalDate payday = day.with(new NextPaydayAdjuster());

        assertEquals("should move to next month", expected, payday);
        assertEquals(DayOfWeek.FRIDAY, payday.getDayOfWeek());
    }

    @Test
    public void checkAdvanceToNextYear_ForDecemberAndDaysAfter15th()
    {
        final LocalDate day = LocalDate.of(2014, Month.DECEMBER, 16);
        final LocalDate expected = LocalDate.of(2015, Month.JANUARY, 23);

        final LocalDate payday = day.with(new NextPaydayAdjuster());

        assertEquals("should move to january", expected, payday);
        assertEquals(DayOfWeek.FRIDAY, payday.getDayOfWeek());
    }

    @Test
    public void checkHandling_Without_Weekend()
    {
        final LocalDate day = LocalDate.of(2015, Month.FEBRUARY, 7);
        final LocalDate expected = LocalDate.of(2015, Month.FEBRUARY, 25);

        final LocalDate payday = day.with(new NextPaydayAdjuster());

        assertEquals("should move to 25. february", expected, payday);
        assertEquals(DayOfWeek.WEDNESDAY, payday.getDayOfWeek());
    }

    @Test
    public void checkHandling_Without_Weekend_ForDecember()
    {
        final LocalDate day = LocalDate.of(2015, Month.DECEMBER, 7);
        final LocalDate expected = LocalDate.of(2015, Month.DECEMBER, 15);

        final LocalDate payday = day.with(new NextPaydayAdjuster());

        assertEquals("should move to 15. december", expected, payday);
        assertEquals(DayOfWeek.TUESDAY, payday.getDayOfWeek());
    }
    
    // all in one
    
    @ParameterizedTest(name="adjustToPayday({0}) => {1}, {2}")
    @CsvSource(value= { "2020-06-21; 2020-06-25; normale Anpassung",
                        "2020-05-27; 2020-06-25; normale Anpassung " + 
                                                "auf nchsten Monat",
                        "2020-07-21; 2020-07-24; Freitag, falls 25. " + 
                                                "am Wochenende",
                        "2019-12-06; 2019-12-16; Dezember: Mitte Monat " + 
                                                "und Montag nach Wochenende",
                        "2019-12-23; 2020-01-24; nchster Monat und Freitag, " +
                                                "falls 25. am Wochenende"}, 
               delimiterString=";")
    void adjustInto(LocalDate startDay, LocalDate expected, String info)
    {
        final TemporalAdjuster paydayAdjuster = new NextPaydayAdjuster();
        
        final Temporal result = paydayAdjuster.adjustInto(startDay);
        
        assertEquals(expected, result);
    }
}

